/*
  Simple MPI program that demonstrates scatter/gather operations

  Jim Teresco
  Department of Computer Science
  Williams College

  Created: Fall 2000

  Last Modification:
  Mon Mar  3 21:55:57 EST 2003

  $Id$
*/

#include <stdio.h>
#include <stdlib.h>
#include <mpi.h>
#include <math.h>

/* initialization of data - master proc only */
void initialize_data(double *data, int numitems) {
  int i;

  for (i=0; i<numitems; i++) {
    data[i] = i+1;
  }
}

/* in a real program this would be more than a single little computation */
double compute_answer(double val) {

  return val*val+sqrt(val)-exp(val);
}

/* print out answers */
void print_answers(double *data, int numitems) {
  int i;

  for (i=0; i<numitems; i++) {
    printf("Got %10.5f from %d\n", data[i], i);
  }
}

int main(int argc, char *argv[]) {
  int rc, i;
  int mypid, numprocs;
  double *data;
  double *answers;
  double mydata, myanswer;

  rc = MPI_Init(&argc,&argv);
  if (rc != MPI_SUCCESS) {
    fprintf(stderr,"Cannot initialize MPI\n");
    MPI_Abort(MPI_COMM_WORLD, 1);
  }

  MPI_Comm_rank(MPI_COMM_WORLD,&mypid);
  MPI_Comm_size(MPI_COMM_WORLD,&numprocs);

  if (mypid == 0) {
    data=(double *)malloc(numprocs*sizeof(double));
    answers=(double *)malloc(numprocs*sizeof(double));

    /* initialize data */
    initialize_data(data, numprocs);
  }

  /* scatter the data to the processors - each gets one double */
  MPI_Scatter(data,1,MPI_DOUBLE,&mydata,1,MPI_DOUBLE,0,MPI_COMM_WORLD);

  /* each processor operates on its data */
  myanswer=compute_answer(mydata);

  /* gather back the answers to the root - one double from each */
  MPI_Gather(&myanswer,1,MPI_DOUBLE,answers,1,MPI_DOUBLE,0,MPI_COMM_WORLD);

  if (mypid == 0) {
    /* root only prints/writes the answers */
    print_answers(answers, numprocs);
    free(data); free(answers);
  }

  MPI_Finalize();
}

